local OI = {}

-- Lua 5.1+ base64 v3.0 (c) 2009 by Alex Kloss <alexthkloss@web.de>
-- licensed under the terms of the LGPL2
 
-- character table string
local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
 
-- encoding
local function enc(data)
    return ((data:gsub('.', function(x) 
        local r,b='',x:byte()
        for i=8,1,-1 do r=r..(b%2^i-b%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return b:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end
	
	-------
local isArray  = { __tostring = function() return "JSON array"  end }    isArray.__index  = isArray
local isObject = { __tostring = function() return "JSON object" end }    isObject.__index = isObject


local function newArray(tbl)
   return setmetatable(tbl or {}, isArray)
end

local function newObject(tbl)
   return setmetatable(tbl or {}, isObject)
end

local function unicode_codepoint_as_utf8(codepoint)
   if codepoint <= 127 then
      return string.char(codepoint)

   elseif codepoint <= 2047 then
      local highpart = math.floor(codepoint / 0x40)
      local lowpart  = codepoint - (0x40 * highpart)
      return string.char(0xC0 + highpart,
                         0x80 + lowpart)

   elseif codepoint <= 65535 then
      local highpart  = math.floor(codepoint / 0x1000)
      local remainder = codepoint - 0x1000 * highpart
      local midpart   = math.floor(remainder / 0x40)
      local lowpart   = remainder - 0x40 * midpart

      highpart = 0xE0 + highpart
      midpart  = 0x80 + midpart
      lowpart  = 0x80 + lowpart
      if ( highpart == 0xE0 and midpart < 0xA0 ) or
         ( highpart == 0xED and midpart > 0x9F ) or
         ( highpart == 0xF0 and midpart < 0x90 ) or
         ( highpart == 0xF4 and midpart > 0x8F )
      then
         return "?"
      else
         return string.char(highpart,
                            midpart,
                            lowpart)
      end

   else
      local highpart  = math.floor(codepoint / 0x40000)
      local remainder = codepoint - 0x40000 * highpart
      local midA      = math.floor(remainder / 0x1000)
      remainder       = remainder - 0x1000 * midA
      local midB      = math.floor(remainder / 0x40)
      local lowpart   = remainder - 0x40 * midB

      return string.char(0xF0 + highpart,
                         0x80 + midA,
                         0x80 + midB,
                         0x80 + lowpart)
   end
end

local function onDecodeError(message, text, location, etc)
   if text then
      if location then
         message = string.format("%s at char %d of: %s", message, location, text)
      else
         message = string.format("%s: %s", message, text)
      end
   end

   if etc ~= nil then
      message = message .. " (" .. encode(etc) .. ")"
   end

   if self.assert then
      self.assert(false, message)
   else
      assert(false, message)
   end
end

onDecodeOfNilError  = onDecodeError
onDecodeOfHTMLError = onDecodeError

local function onEncodeError(message, etc)
   if etc ~= nil then
      message = message .. " (" .. encode(etc) .. ")"
   end

   if self.assert then
      self.assert(false, message)
   else
      assert(false, message)
   end
end

local function grok_number(self, text, start, etc)
   local integer_part = text:match('^-?[1-9]%d*', start)
                     or text:match("^-?0",        start)

   if not integer_part then
      self:onDecodeError("expected number", text, start, etc)
   end

   local i = start + integer_part:len()
   local decimal_part = text:match('^%.%d+', i) or ""

   i = i + decimal_part:len()
   local exponent_part = text:match('^[eE][-+]?%d+', i) or ""

   i = i + exponent_part:len()

   local full_number_text = integer_part .. decimal_part .. exponent_part
   local as_number = tonumber(full_number_text)

   if not as_number then
      self:onDecodeError("bad number", text, start, etc)
   end

   return as_number, i
end


local function grok_string(self, text, start, etc)

   if text:sub(start,start) ~= '"' then
      self:onDecodeError("expected string's opening quote", text, start, etc)
   end

   local i = start + 1 -- +1 to bypass the initial quote
   local text_len = text:len()
   local VALUE = ""
   while i <= text_len do
      local c = text:sub(i,i)
      if c == '"' then
         return VALUE, i + 1
      end
      if c ~= '\\' then
         VALUE = VALUE .. c
         i = i + 1
      elseif text:match('^\\b', i) then
         VALUE = VALUE .. "\b"
         i = i + 2
      elseif text:match('^\\f', i) then
         VALUE = VALUE .. "\f"
         i = i + 2
      elseif text:match('^\\n', i) then
         VALUE = VALUE .. "\n"
         i = i + 2
      elseif text:match('^\\r', i) then
         VALUE = VALUE .. "\r"
         i = i + 2
      elseif text:match('^\\t', i) then
         VALUE = VALUE .. "\t"
         i = i + 2
      else
         local hex = text:match('^\\u([0123456789aAbBcCdDeEfF][0123456789aAbBcCdDeEfF][0123456789aAbBcCdDeEfF][0123456789aAbBcCdDeEfF])', i)
         if hex then
            i = i + 6
local codepoint = tonumber(hex, 16)
            if codepoint >= 0xD800 and codepoint <= 0xDBFF then
               local lo_surrogate = text:match('^\\u([dD][cdefCDEF][0123456789aAbBcCdDeEfF][0123456789aAbBcCdDeEfF])', i)
               if lo_surrogate then
                  i = i + 6 -- bypass the low surrogate we just read
                  codepoint = 0x2400 + (codepoint - 0xD800) * 0x400 + tonumber(lo_surrogate, 16)
               else
               end
            end
            VALUE = VALUE .. unicode_codepoint_as_utf8(codepoint)

         else

            VALUE = VALUE .. text:match('^\\(.)', i)
            i = i + 2
         end
      end
   end

   self:onDecodeError("unclosed string", text, start, etc)
end

local function skip_whitespace(text, start)

   local _, match_end = text:find("^[ \n\r\t]+", start) -- [http://www.ietf.org/rfc/rfc4627.txt] Section 2
   if match_end then
      return match_end + 1
   else
      return start
   end
end

local grok_one -- assigned later

local function grok_object(self, text, start, etc)
   if text:sub(start,start) ~= '{' then
      self:onDecodeError("expected '{'", text, start, etc)
   end

   local i = skip_whitespace(text, start + 1) -- +1 to skip the '{'

   local VALUE = self.strictTypes and self:newObject { } or { }

   if text:sub(i,i) == '}' then
      return VALUE, i + 1
   end
   local text_len = text:len()
   while i <= text_len do
      local key, new_i = grok_string(self, text, i, etc)

      i = skip_whitespace(text, new_i)

      if text:sub(i, i) ~= ':' then
         self:onDecodeError("expected colon", text, i, etc)
      end

      i = skip_whitespace(text, i + 1)

      local new_val, new_i = grok_one(self, text, i)

      VALUE[key] = new_val

      --
      -- Expect now either '}' to end things, or a ',' to allow us to continue.
      --
      i = skip_whitespace(text, new_i)

      local c = text:sub(i,i)

      if c == '}' then
         return VALUE, i + 1
      end

      if text:sub(i, i) ~= ',' then
         self:onDecodeError("expected comma or '}'", text, i, etc)
      end

      i = skip_whitespace(text, i + 1)
   end

   self:onDecodeError("unclosed '{'", text, start, etc)
end

local function grok_array(self, text, start, etc)
   if text:sub(start,start) ~= '[' then
      self:onDecodeError("expected '['", text, start, etc)
   end

   local i = skip_whitespace(text, start + 1) -- +1 to skip the '['
   local VALUE = self.strictTypes and self:newArray { } or { }
   if text:sub(i,i) == ']' then
      return VALUE, i + 1
   end

   local VALUE_INDEX = 1

   local text_len = text:len()
   while i <= text_len do
      local val, new_i = grok_one(self, text, i)

      -- can't table.insert(VALUE, val) here because it's a no-op if val is nil
      VALUE[VALUE_INDEX] = val
      VALUE_INDEX = VALUE_INDEX + 1

      i = skip_whitespace(text, new_i)

      --
      -- Expect now either ']' to end things, or a ',' to allow us to continue.
      --
      local c = text:sub(i,i)
      if c == ']' then
         return VALUE, i + 1
      end
      if text:sub(i, i) ~= ',' then
         self:onDecodeError("expected comma or '['", text, i, etc)
      end
      i = skip_whitespace(text, i + 1)
   end
   self:onDecodeError("unclosed '['", text, start, etc)
end


grok_one = function(self, text, start, etc)
   -- Skip any whitespace
   start = skip_whitespace(text, start)

   if start > text:len() then
      self:onDecodeError("unexpected end of string", text, nil, etc)
   end

   if text:find('^"', start) then
      return grok_string(self, text, start, etc)

   elseif text:find('^[-0123456789 ]', start) then
      return grok_number(self, text, start, etc)

   elseif text:find('^%{', start) then
      return grok_object(self, text, start, etc)

   elseif text:find('^%[', start) then
      return grok_array(self, text, start, etc)

   elseif text:find('^true', start) then
      return true, start + 4

   elseif text:find('^false', start) then
      return false, start + 5

   elseif text:find('^null', start) then
      return nil, start + 4

   else
      self:onDecodeError("can't parse JSON", text, start, etc)
   end
end



local function backslash_replacement_function(c)
   if c == "\n" then
      return "\\n"
   elseif c == "\r" then
      return "\\r"
   elseif c == "\t" then
      return "\\t"
   elseif c == "\b" then
      return "\\b"
   elseif c == "\f" then
      return "\\f"
   elseif c == '"' then
      return '\\"'
   elseif c == '\\' then
      return '\\\\'
   else
      return string.format("\\u%04x", c:byte())
   end
end

local chars_to_be_escaped_in_JSON_string
   = '['
   ..    '"'    -- class sub-pattern to match a double quote
   ..    '%\\'  -- class sub-pattern to match a backslash
   ..    '%z'   -- class sub-pattern to match a null
   ..    '\001' .. '-' .. '\031' -- class sub-pattern to match control characters
   .. ']'

local function json_string_literal(value)
   local newval = value:gsub(chars_to_be_escaped_in_JSON_string, backslash_replacement_function)
   return '"' .. newval .. '"'
end

local function object_or_array(self, T, etc)

   local string_keys = { }
   local number_keys = { }
   local number_keys_must_be_strings = false
   local maximum_number_key

   for key in pairs(T) do
      if type(key) == 'string' then
         table.insert(string_keys, key)
      elseif type(key) == 'number' then
         table.insert(number_keys, key)
         if key <= 0 or key >= math.huge then
            number_keys_must_be_strings = true
         elseif not maximum_number_key or key > maximum_number_key then
            maximum_number_key = key
         end
      else
         self:onEncodeError("can't encode table with a key of type " .. type(key), etc)
      end
   end

   if #string_keys == 0 and not number_keys_must_be_strings then
      --
      -- An empty table, or a numeric-only array
      --
      if #number_keys > 0 then
         return nil, maximum_number_key -- an array
      elseif tostring(T) == "JSON array" then
         return nil
      elseif tostring(T) == "JSON object" then
         return { }
      else
         -- have to guess, so we'll pick array, since empty arrays are likely more common than empty objects
         return nil
      end
   end

   table.sort(string_keys)

   local map
   if #number_keys > 0 then
      --
      -- If we're here then we have either mixed string/number keys, or numbers inappropriate for a JSON array
      -- It's not ideal, but we'll turn the numbers into strings so that we can at least create a JSON object.
      --

      --if self.noKeyConversion then
      --   self:onEncodeError("a table with both numeric and string keys could be an object or array; aborting", etc)
      --end

      --
      -- Have to make a shallow copy of the source table so we can remap the numeric keys to be strings
      --
      map = { }
      for key, val in pairs(T) do
         map[key] = val
      end

      table.sort(number_keys)

      --
      -- Throw numeric keys in there as strings
      --
      for _, number_key in ipairs(number_keys) do
         local string_key = tostring(number_key)
         if map[string_key] == nil then
            table.insert(string_keys , string_key)
            map[string_key] = T[number_key]
         else
            self:onEncodeError("conflict converting table with mixed-type keys into a JSON object: key " .. number_key .. " exists both as a string and a number.", etc)
         end
      end
   end

   return string_keys, nil, map
end

--
-- Encode
--
local encode_value -- must predeclare because it calls itself
function encode_value(self, value, parents, etc, indent) -- non-nil indent means pretty-printing

   if value == nil then
      return 'null'

   elseif type(value) == 'string' then
      return json_string_literal(value)

   elseif type(value) == 'number' then
      if value ~= value then
 
         return "null"
      elseif value >= math.huge then

         return "1e+9999"
      elseif value <= -math.huge then
         return "-1e+9999"
      else
         return tostring(value)
      end

   elseif type(value) == 'boolean' then
      return tostring(value)

   elseif type(value) ~= 'table' then
      self:onEncodeError("can't convert " .. type(value) .. " to JSON", etc)

   else
      local T = value

      if parents[T] then
         self:onEncodeError("table " .. tostring(T) .. " is a child of itself", etc)
      else
         parents[T] = true
      end

      local result_value

      local object_keys, maximum_number_key, map = object_or_array(self, T, etc)
      if maximum_number_key then
         --
         -- An array...
         --
         local ITEMS = { }
         for i = 1, maximum_number_key do
            table.insert(ITEMS, encode_value(self, T[i], parents, etc, indent))
         end

         if indent then
            result_value = "[ " .. table.concat(ITEMS, ", ") .. " ]"
         else
            result_value = "[" .. table.concat(ITEMS, ",") .. "]"
         end

      elseif object_keys then
         --
         -- An object
         --
         local TT = map or T

         if indent then

            local KEYS = { }
            local max_key_length = 0
            for _, key in ipairs(object_keys) do
               local encoded = encode_value(self, tostring(key), parents, etc, "")
               max_key_length = math.max(max_key_length, #encoded)
               table.insert(KEYS, encoded)
            end
            local key_indent = indent .. "    "
            local subtable_indent = indent .. string.rep(" ", max_key_length + 2 + 4)
            local FORMAT = "%s%" .. string.format("%d", max_key_length) .. "s: %s"

            local COMBINED_PARTS = { }
            for i, key in ipairs(object_keys) do
               local encoded_val = encode_value(self, TT[key], parents, etc, subtable_indent)
               table.insert(COMBINED_PARTS, string.format(FORMAT, key_indent, KEYS[i], encoded_val))
            end
            result_value = "{\n" .. table.concat(COMBINED_PARTS, ",\n") .. "\n" .. indent .. "}"

         else

            local PARTS = { }
            for _, key in ipairs(object_keys) do
               local encoded_val = encode_value(self, TT[key],       parents, etc, indent)
               local encoded_key = encode_value(self, tostring(key), parents, etc, indent)
               table.insert(PARTS, string.format("%s:%s", encoded_key, encoded_val))
            end
            result_value = "{" .. table.concat(PARTS, ",") .. "}"

         end
      else
         --
         -- An empty array/object... we'll treat it as an array, though it should really be an option
         --
         result_value = "[]"
      end

      parents[T] = false
      return result_value
   end
end


local function encode(value, etc)
   return encode_value(self, value, {}, etc, nil)
end

local function encode_pretty(value, etc)
   return encode_value(self, value, {}, etc, "")
end
 

local function new(args)
   local new = { }

   if args then
      for key, val in pairs(args) do
         new[key] = val
      end
   end

   return setmetatable(new)
end

------------------------------
OI.name = "OghmaInfinium"
OI.Data = {}
OI.URL = {}
OI.URL.protocol = "1"
OI.URL.endpoint = "input.php"
OI.URL.server = "http://OghmaInfinium.com/"
OI.URL.stream = "stream"
OI.version = "0.109"
OI.CMD = "/openoi"
OICodedData = {}
local AccountDefaults = {}
AccountDefaults = {
	User = "Username",
	Pass = "Password"
}
local function round(n)
  return math.floor((math.floor(n*2) + 1)/2)
end
function url_encode(str)
  if (str) then
    str = string.gsub (str, "\n", "\r\n")
    str = string.gsub (str, "([^%w %-%_%.%~])",
        function (c) return string.format ("%%%02X", string.byte(c)) end)
    --str = string.gsub (str, " ", "%2")
  end
  return str	
end
function OI:GetandEncode()
	local DataLore = url_encode(enc(url_encode(encode(OI.Data)))) -- We will never forget May 23, 1994
	--one = string.sub(str, 1, 1023)

	arr = {}
	lenght = string.len(DataLore) / 1024 + 1
	for i=1, lenght do
		if i == 1 then
			arr[i] = string.sub(DataLore, 1, 1023)
		else
			temp = string.sub(DataLore, 1024)
			temp1 = temp
			if string.len(temp) > 1023 then
				temp1 = string.sub(temp, 0, 1023)
			end
			DataLore = temp
			arr[i] = temp1
		end
		i = i+1
	end
	OICodedData.savedVariables.Json = arr
	return url_encode(enc(url_encode(encode(OI.Data))))
end
function OI:ReturnNope()
	return "Not", "Enough", "room", "Please use external application"
end
function OI:FetchURL()
	local str = OI:GetandEncode()
	arr = {}
	length = string.len(str) / 1024 + 1
	if length >= 5 then
		return OI:ReturnNope()
	else
		for i=1, length do
			if i == 1 then
				arr[i] = string.sub(str, 1, 1023)
			else
				temp = string.sub(str, 1024)
				temp1 = temp
				if string.len(temp) > 1023 then
					temp1 = string.sub(temp, 0, 1023)
				end
				str = temp
				arr[i] = temp1		
			end
			i = i+1
		end
	end
	return  arr[1], arr[2], arr[3], arr[4]
end

function OI:Update()
	OI.Data.aliance = GetUnitAlliance("player")
	OI.Data.class = GetUnitClassId("player")
	OI.Data.name = UndecorateDisplayName(GetDisplayName())
	OI.Data.charname = GetUnitName("player")
	OI.Data.level = GetUnitLevel("player")
	OI.Data.XP = GetUnitXP("player").."/"..GetUnitXPMax("player")
	OI.Data.Race = GetUnitRace("player")
	OI.Data.username = OI.UI.login_u:GetText()
	OI.Data.password = OI.UI.login_p:GetText()
	OI.Data.Title = GetUnitTitle("player")
	OI.Data.Stats = {}
	OI.Data.Stats.Raw = {}
	OI.Data.Stats.Raw.Max_Health = GetPlayerStat(STAT_HEALTH_MAX, STAT_BONUS_OPTION_DONT_APPLY_BONUS, STAT_SOFT_CAP_OPTION_APPLY_SOFT_CAP)
	OI.Data.Stats.Raw.Max_Magicka =   GetPlayerStat(STAT_MAGICKA_MAX , STAT_BONUS_OPTION_DONT_APPLY_BONUS, STAT_SOFT_CAP_OPTION_APPLY_SOFT_CAP)
	OI.Data.Stats.Raw.Max_Stamina = GetPlayerStat(STAT_STAMINA_MAX, STAT_BONUS_OPTION_DONT_APPLY_BONUS, STAT_SOFT_CAP_OPTION_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed = {}
	OI.Data.Stats.Buffed.Max_Health = GetPlayerStat(STAT_HEALTH_MAX, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Max_Magicka =   GetPlayerStat(STAT_MAGICKA_MAX , STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Max_Stamina = GetPlayerStat(STAT_STAMINA_MAX, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Spell_Crit = GetPlayerStat(STAT_SPELL_CRITICAL, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Spell_Damage =  GetPlayerStat(STAT_SPELL_POWER, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Weapon_Damage = GetPlayerStat(STAT_WEAPON_POWER, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Weapon_Crit = GetPlayerStat(STAT_CRITICAL_STRIKE, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Health_Regen_Combat = GetPlayerStat(STAT_HEALTH_REGEN_COMBAT, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Health_Regen_Idle = GetPlayerStat(STAT_HEALTH_REGEN_IDLE, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Magicka_Regen_Combat = GetPlayerStat(STAT_MAGICKA_REGEN_COMBAT, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Magicka_Regen_Idle = GetPlayerStat(STAT_MAGICKA_REGEN_IDLE, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Stamina_Regen_Combat = GetPlayerStat(STAT_STAMINA_REGEN_COMBAT, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	OI.Data.Stats.Buffed.Stamina_Regen_Idle = GetPlayerStat(STAT_STAMINA_REGEN_IDLE, STAT_BONUS_OPTION_APPLY_BONUS, STAT_SOFT_CAP_OPTION_DONT_APPLY_SOFT_CAP)
	
	OI.Data.Stats.Buffs = {}
	OI.Data.Stats.Buffsamount = GetNumBuffs("player")
	for i=1,OI.Data.Stats.Buffsamount do
		OI.Data.Stats.Buffs[i] = {}
		OI.Data.Stats.Buffs[i].buffName, OI.Data.Stats.Buffs[i].timeStarted, OI.Data.Stats.Buffs[i].timeEnding, OI.Data.Stats.Buffs[i].buffSlot, OI.Data.Stats.Buffs[i].stackCount, OI.Data.Stats.Buffs[i].iconFilename, OI.Data.Stats.Buffs[i].buffType, OI.Data.Stats.Buffs[i].effectType, OI.Data.Stats.Buffs[i].abilityType, OI.Data.Stats.Buffs[i].statusEffectType = GetUnitBuffInfo("player", i)
	end
	
	OI.Data.General = {}
	OI.Data.General.TimeStamp = GetTimeStamp() -- id64.
	OI.Data.Achievements = {} 
	OI.Data.Achievements.EarnedPoints = GetEarnedAchievementPoints()
	OI.Data.Achievements.TotalPoints = GetTotalAchievementPoints()

	OI.Data.Bags = {}
	OI.Data.Bags.BAG_WORN = {}
	OI.Data.Zone = GetUnitZone("player")
	local used = GetNumBagUsedSlots(BAG_WORN)
	
	local worn_size = GetBagSize(BAG_WORN)
  
	for i=1,worn_size do
		local Name = GetItemName(BAG_WORN,i)
		if string.len(Name) >= 1 then
			OI.Data.Bags.BAG_WORN[i] = {}
			OI.Data.Bags.BAG_WORN[i].Name = GetItemName(BAG_WORN,i)
			OI.Data.Bags.BAG_WORN[i].Link = GetItemLink(BAG_WORN,i,LINK_STYLE_BRACKETS)
			local icon, stack, sellPrice, meetsUsageRequirement, locked, equipType, itemStyle, quality = GetItemInfo(BAG_WORN,i)
			OI.Data.Bags.BAG_WORN[i].Icon = icon
			OI.Data.Bags.BAG_WORN[i].Style = itemStyle
			OI.Data.Bags.BAG_WORN[i].Quality = quality
			OI.Data.Bags.BAG_WORN[i].Slot = i
		end
	end

  
	local one, two, three, four = OI:FetchURL()
	OI.UI.link:SetText(one)
	OI.UI.link1:SetText(two)
	OI.UI.link2:SetText(three)
	OI.UI.link3:SetText(four)
	OICodedData.Account.User = OI.UI.login_u:GetText()
	OICodedData.Account.Pass = OI.UI.login_p:GetText()
end
function OI:open()
	OI.UI:SetHidden(false)
end
function OI:close()
	OI.UI.link:SetText( "" )
	OI.UI:SetHidden(true)
end
function OI.toggle()
	if (OI.UI:IsHidden()) then
		OI.open()
	else
		OI.close()
	end
end
function OI:Create()
	local wm = GetWindowManager()
	local height = GuiRoot:GetHeight()
	local width = GuiRoot:GetWidth()

	OI.UI = OghmaInfiniumUI
   
   
	OI.UI.bd = OghmaInfiniumUI_BD
	OI.UI.bd:SetCenterColor(0,0,0,.5)
	OI.UI.bd:SetEdgeColor(0,0,0,.2)
	OI.UI.bd:SetEdgeTexture("", 8, 1, 1)


	OI.UI.title1 = OghmaInfiniumUI_Title
	OI.UI.title1:SetText(OI.name)

	OI.UI.title2 = OghmaInfiniumUI_VersionTitle
	OI.UI.title2:SetText("|cAAAAAA" .. "Version ".. OI.version .. "|r")

	OI.UI.link_title = OghmaInfiniumUI_LinkTitle
	OI.UI.link_title:SetText("Use the external application, or copy and paste this to the update page.")
	
	OI.UI.link_info = OghmaInfiniumUI_Info
	
	OI.UI.link_bg = wm:CreateControlFromVirtual("$(parent)LinkBG", OI.UI, "ZO_EditBackdrop")
	OI.UI.link_bg:SetDimensions(500, 100)
	OI.UI.link_bg:SetAnchor(TOPLEFT, OI.UI.bd, TOPLEFT, 10, 60 )
	
	OI.UI.link = wm:CreateControlFromVirtual("$(parent)Link", OI.UI, "ZO_DefaultEditForBackdrop")
	OI.UI.link:SetAnchorFill( OI.UI.link_bg )
	OI.UI.link:SetMaxInputChars( 2000 )
	OI.UI.link:SetMultiLine(true)
	OI.UI.link:SetEditEnabled( false )
	OI.UI.link:SelectAll()
	OI.UI.link:SetCursorPosition(0)
	OI.UI.link:TakeFocus()
	
	OI.UI.link_bg1 = wm:CreateControlFromVirtual("$(parent)LinkBG1", OI.UI, "ZO_EditBackdrop")
	OI.UI.link_bg1:SetDimensions(500, 100)
	OI.UI.link_bg1:SetAnchor(TOPLEFT, OI.UI.bd, TOPLEFT, 10, 170 )
	
	OI.UI.link1 = wm:CreateControlFromVirtual("$(parent)Link1", OI.UI, "ZO_DefaultEditForBackdrop")
	OI.UI.link1:SetAnchorFill( OI.UI.link_bg1 )
	OI.UI.link1:SetMaxInputChars( 2000 )
	OI.UI.link1:SetMultiLine(true)
	OI.UI.link1:SetEditEnabled( false )
	OI.UI.link1:SelectAll()
	OI.UI.link1:SetCursorPosition(0)
	OI.UI.link1:TakeFocus()
	
	OI.UI.link_bg2 = wm:CreateControlFromVirtual("$(parent)LinkBG2", OI.UI, "ZO_EditBackdrop")
	OI.UI.link_bg2:SetDimensions(500, 100)
	OI.UI.link_bg2:SetAnchor(TOPLEFT, OI.UI.bd, TOPLEFT, 10, 280 )
	
	OI.UI.link2 = wm:CreateControlFromVirtual("$(parent)Link2", OI.UI, "ZO_DefaultEditForBackdrop")
	OI.UI.link2:SetAnchorFill( OI.UI.link_bg2 )
	OI.UI.link2:SetMaxInputChars( 2000 )
	OI.UI.link2:SetMultiLine(true)
	OI.UI.link2:SetEditEnabled( false )
	OI.UI.link2:SelectAll()
	OI.UI.link2:SetCursorPosition(0)
	OI.UI.link2:TakeFocus()
         
	OI.UI.link_bg3 = wm:CreateControlFromVirtual("$(parent)LinkBG3", OI.UI, "ZO_EditBackdrop")
	OI.UI.link_bg3:SetDimensions(500, 100)
	OI.UI.link_bg3:SetAnchor(TOPLEFT, OI.UI.bd, TOPLEFT, 10, 390 )

	OI.UI.link3 = wm:CreateControlFromVirtual("$(parent)Link3", OI.UI, "ZO_DefaultEditForBackdrop")
	OI.UI.link3:SetAnchorFill( OI.UI.link_bg3 )
	OI.UI.link3:SetMaxInputChars( 2000 )
	OI.UI.link3:SetMultiLine(true)
	OI.UI.link3:SetEditEnabled( false )
	OI.UI.link3:SelectAll()
	OI.UI.link3:SetCursorPosition(0)
	OI.UI.link3:TakeFocus()
	
	OI.UI.login_bg = wm:CreateControlFromVirtual("$(parent)login_bg", OI.UI, "ZO_EditBackdrop")
	OI.UI.login_bg:SetDimensions(100, 20)
	OI.UI.login_bg:SetAnchor(BOTTOMLEFT, OI.UI.bd, BOTTOMLEFT, 0, 0 )
	OI.UI.login_u = wm:CreateControlFromVirtual("$(parent)login_u", OI.UI, "ZO_DefaultEditForBackdrop")
	OI.UI.login_u:SetAnchorFill( OI.UI.login_bg )
	OI.UI.login_u:SetMaxInputChars( 100 )
	OI.UI.login_u:SetMultiLine(false)
	OI.UI.login_u:SetEditEnabled( true )
	OI.UI.login_u:SetText(OICodedData.Account.User)
	OI.UI.login_bg1 = wm:CreateControlFromVirtual("$(parent)login_bg1", OI.UI, "ZO_EditBackdrop")
	OI.UI.login_bg1:SetDimensions(100, 20)
	OI.UI.login_bg1:SetAnchor(BOTTOMRIGHT, OI.UI.bd, BOTTOMRIGHT, 0, 0 )
	OI.UI.login_p = wm:CreateControlFromVirtual("$(parent)login_p", OI.UI, "ZO_DefaultEditForBackdrop")
	OI.UI.login_p:SetAnchorFill( OI.UI.login_bg1 )
	OI.UI.login_p:SetMaxInputChars( 100 )
	OI.UI.login_p:SetMultiLine(false)
	OI.UI.login_p:SetEditEnabled( true )
	OI.UI.login_p:SetTextType(TEXT_TYPE_PASSWORD)
	OI.UI.login_p:SetText(OICodedData.Account.Pass)
	
	OI.UI.btnUpdate = wm:CreateControlFromVirtual("$(parent)Update", OI.UI, "ZO_DefaultButton")
	OI.UI.btnUpdate:SetAnchor(BOTTOMLEFT, OI.UI.bd, BOTTOMLEFT, 0, 30)
	OI.UI.btnUpdate:SetWidth(170)
	OI.UI.btnUpdate:SetText('Create a new link')
	OI.UI.btnUpdate:SetHandler( "OnClicked", function() OI:Update() end )
	
	OI.UI.btnClose = wm:CreateControlFromVirtual("$(parent)Close", OI.UI, "ZO_DefaultButton")
	OI.UI.btnClose:SetAnchor(BOTTOMLEFT, OI.UI.bd, BOTTOMLEFT, 350, 30 )
	OI.UI.btnClose:SetWidth(170)
	OI.UI.btnClose:SetText('Close')
	OI.UI.btnClose:SetHandler( "OnClicked",function() OI:close() end )
	
	
	OI.toggleButtonWindow = wm:CreateTopLevelWindow( "OIUITOGGLE" )
	OI.toggleButtonWindow.toggleButton = CreateControlFromVirtual("$(parent)BUTTON", OI.toggleButtonWindow, "ZO_DefaultButton")
	OI.toggleButtonWindow.toggleButton:SetDimensions(195,31)
	OI.toggleButtonWindow.toggleButton:SetHidden(false)
	OI.toggleButtonWindow.toggleButton:SetFont("ZoFontGameBold")
	OI.toggleButtonWindow.toggleButton:SetVerticalAlignment(TEXT_ALIGN_CENTER)
	OI.toggleButtonWindow.toggleButton:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
	OI.toggleButtonWindow.toggleButton:SetText( 'Oghma Infinium' )
	OI.toggleButtonWindow.toggleButton:SetAnchor(BOTTOMRIGHT, GuiRoot, BOTTOMRIGHT, -300, -16)
	OI.toggleButtonWindow.toggleButton:SetHandler("OnMouseDown", function() OI.toggle() end)
		

end
function OI:Initialize()	
	OI:Create()
	SLASH_COMMANDS[OI.CMD] = OI.toggle
end
 function OI.FCMD(t)
	if (t == "") then
		if ( OI.toggleButtonWindow.toggleButton:IsHidden() ) then
			OI.toggleButtonWindow.toggleButton:SetHidden(false)
		else
			OI.toggleButtonWindow.toggleButton:SetHidden(true)
		end
	end
end
function OI.OnAddOnLoaded(event, addonName)
	if addonName == OI.name then
		OICodedData.savedVariables = ZO_SavedVars:New("OICodedData", 0.08, savedVariables, nil)
		OICodedData.Account = ZO_SavedVars:NewAccountWide("OICodedData", 0.08, Account, AccountDefaults)
		OI:Initialize()
	end
end
EVENT_MANAGER:RegisterForEvent(OI.name, EVENT_ADD_ON_LOADED, OI.OnAddOnLoaded)